Attribute VB_Name = "MarchingCubeGeneration"
' the main marching cube generation algoritm
Private Function CheckLimit(Limit As Single, value As Single) As Integer
' kontrola zda je roh krychle vetsi nez limit
If value > Limit Then
    CheckLimit = 1
    Else: CheckLimit = 0
End If
End Function
Private Function Interpolace(a As Single, b As Single, Limit As Single) As Single
    ' interpolace na stred hrany ( v rozmezi 0-1)
    Interpolace = (a - Limit) / (a - b)
End Function

Private Sub GeneratePruseciky(Pruseciky() As XYZ, _
StartPoint As XYZ, _
EdgeTopology, grid As GRIDCEL, Limit As Single, VoxelDescription, _
index As Integer)
' generovani pozic na hranach krychle
Dim i As Integer
Dim edge As Integer

For i = 1 To EdgeTopology(index, 0)
Select Case EdgeTopology(index, i)

    Case 1
        Pruseciky(1).x = StartPoint.x + VoxelDescription(7) * Interpolace(grid.value(0), grid.value(1), Limit)
        Pruseciky(1).y = StartPoint.y
        Pruseciky(1).z = StartPoint.z
    Case 2
        Pruseciky(2).x = StartPoint.x + VoxelDescription(7)
        Pruseciky(2).y = StartPoint.y + VoxelDescription(8) * Interpolace(grid.value(1), grid.value(2), Limit)
        Pruseciky(2).z = StartPoint.z
    Case 3
        Pruseciky(3).x = StartPoint.x + VoxelDescription(7) * Interpolace(grid.value(3), grid.value(2), Limit)
        Pruseciky(3).y = StartPoint.y + VoxelDescription(8)
        Pruseciky(3).z = StartPoint.z
    Case 4
        Pruseciky(4).x = StartPoint.x
        Pruseciky(4).y = StartPoint.y + VoxelDescription(7) * Interpolace(grid.value(0), grid.value(3), Limit)
        Pruseciky(4).z = StartPoint.z
    Case 5
        Pruseciky(5).x = StartPoint.x + VoxelDescription(7) * Interpolace(grid.value(4), grid.value(5), Limit)
        Pruseciky(5).y = StartPoint.y
        Pruseciky(5).z = StartPoint.z + VoxelDescription(9)
    Case 6
        Pruseciky(6).x = StartPoint.x + VoxelDescription(7)
        Pruseciky(6).y = StartPoint.y + VoxelDescription(8) * Interpolace(grid.value(5), grid.value(6), Limit)
        Pruseciky(6).z = StartPoint.z + VoxelDescription(9)
    Case 7
        Pruseciky(7).x = StartPoint.x + VoxelDescription(7) * Interpolace(grid.value(7), grid.value(6), Limit)
        Pruseciky(7).y = StartPoint.y + VoxelDescription(8)
        Pruseciky(7).z = StartPoint.z + VoxelDescription(9)
    Case 8
        Pruseciky(8).x = StartPoint.x
        Pruseciky(8).y = StartPoint.y + VoxelDescription(8) * Interpolace(grid.value(4), grid.value(7), Limit)
        Pruseciky(8).z = StartPoint.z + VoxelDescription(9)
    Case 9
        Pruseciky(9).x = StartPoint.x
        Pruseciky(9).y = StartPoint.y
        Pruseciky(9).z = StartPoint.z + VoxelDescription(9) * Interpolace(grid.value(0), grid.value(4), Limit)
    Case 10
        Pruseciky(10).x = StartPoint.x + VoxelDescription(7)
        Pruseciky(10).y = StartPoint.y
        Pruseciky(10).z = StartPoint.z + VoxelDescription(9) * Interpolace(grid.value(1), grid.value(5), Limit)
    Case 11
        Pruseciky(11).x = StartPoint.x
        Pruseciky(11).y = StartPoint.y + VoxelDescription(8)
        Pruseciky(11).z = StartPoint.z + VoxelDescription(9) * Interpolace(grid.value(3), grid.value(7), Limit)
    Case 12
        Pruseciky(12).x = StartPoint.x + VoxelDescription(7)
        Pruseciky(12).y = StartPoint.y + VoxelDescription(8)
        Pruseciky(12).z = StartPoint.z + VoxelDescription(9) * Interpolace(grid.value(2), grid.value(6), Limit)
    End Select

Next i

End Sub

Public Sub GenerateTriangles(VoxelDescription, Voxel, Limit As Single, _
Layer As Integer, Triangles() As TRIANGLE, NumberOfTriangles As Long)
' --------------------
'MAIN INPUT FUNCTION '
'---------------------
'VoxelDescription 3x Rozmery voxelu, x0,y0,z0,krokx,kroky,krokz
'Voxel - pole x,y,z s prostorovou hodnotou
'Limit - limit vyberu pro kladnou intenzitu
'Triangles pole NumberofTrianglex , 3,3 pro x,y,z,trojuhelniku

Dim CubePoints(1 To 8) As Integer
Dim grid As GRIDCEL
Dim x As Integer
Dim y As Integer
Dim z As Integer
Dim index As Integer
Dim i As Integer
Dim StartPoint As XYZ
Static TriangleTopology(0 To 255, 0 To 30) As Integer
Static EdgeTopology(0 To 255, 0 To 12) As Integer
Static ReadyCubeTopology As Boolean ' identificator if read was done


If Not ReadyCubeTopology Then
  ReadMarchingTopology TriangleTopology, EdgeTopology
  ReadyCubeTopology = True ' topology will be read only once
End If

' cube
'               ^ y z going back
'               8------7
'             4------3 |
'             | 5----|-6
'             1------2    ->x

NumberOfTriangles = 0

' graphical progress outup
frmProgressBar.ProgressBar1.value = 0
frmProgressBar.Caption = "ELD map processing progress. Layer number:" + Str$(Layer)
'frmProgressBar.LaerNumber.Text = Layer
frmProgressBar.Show
frmProgressBar.AutoRedraw = True
For x = 1 To VoxelDescription(1) - 1 'rez
' graphical progress output
frmProgressBar.ProgressBar1.value = 100 * (x / (VoxelDescription(1) - 1))
    For y = 1 To VoxelDescription(2) - 1
        For z = 1 To VoxelDescription(3) - 1
            
            StartPoint.x = (((x - 1) * VoxelDescription(7)) + VoxelDescription(4))
            StartPoint.y = (((y - 1) * VoxelDescription(8)) + VoxelDescription(5))
            StartPoint.z = (((z - 1) * VoxelDescription(9)) + VoxelDescription(6))
               
            grid.value(0) = Voxel(x, y, z)
            'grid.p(0) = StartPoint
                        
            grid.value(1) = Voxel(x + 1, y, z)
            'grid.p(1) = PlusVectorPoint(StartPoint, (VoxelDescription(7)), 0, 0)
            
            grid.value(2) = Voxel(x + 1, y + 1, z)
            'grid.p(2) = PlusVectorPoint(StartPoint, (VoxelDescription(7)), (VoxelDescription(8)), 0)

            grid.value(3) = Voxel(x, y + 1, z)
            'grid.p(3) = PlusVectorPoint(StartPoint, 0, (VoxelDescription(8)), 0)
            
            grid.value(4) = Voxel(x, y, z + 1)
            'grid.p(4) = PlusVectorPoint(StartPoint, 0, 0, (VoxelDescription(9)))
            
            grid.value(5) = Voxel(x + 1, y, z + 1)
            'grid.p(5) = PlusVectorPoint(StartPoint, (VoxelDescription(7)), 0, (VoxelDescription(9)))

            grid.value(6) = Voxel(x + 1, y + 1, z + 1)
            'grid.p(6) = PlusVectorPoint(StartPoint, (VoxelDescription(7)), (VoxelDescription(8)), (VoxelDescription(9)))

            grid.value(7) = Voxel(x, y + 1, z + 1)
            'grid.p(7) = PlusVectorPoint(StartPoint, 0, (VoxelDescription(8)), (VoxelDescription(9)))
         
            
            For i = 1 To 8
                 CubePoints(i) = CheckLimit(Limit, grid.value(i - 1))
            Next i
                        
            ' Vypocet indexu pro urceni topologie krychle
            index = CubePoints(1) + _
            CubePoints(2) * 2 + _
            CubePoints(3) * 4 + _
            CubePoints(4) * 8 + _
            CubePoints(5) * 16 + _
            CubePoints(6) * 32 + _
            CubePoints(7) * 64 + _
            CubePoints(8) * 128


If index = 0 Or index = 255 Then GoTo skip ' zadny nebo zcela zaplneni prostor
            
            GenerateCubeTriangle index, _
            StartPoint, _
            grid, _
            Layer, Triangles, NumberOfTriangles, _
            TriangleTopology, EdgeTopology, _
            Limit, VoxelDescription
            
skip:
                
        Next z
    Next y
Next x
frmProgressBar.Hide
End Sub

Private Sub GenerateCubeTriangle(index As Integer, _
StartPoint As XYZ, grid As GRIDCEL, _
Layer As Integer, Triangles() As TRIANGLE, NumberOfTriangles As Long, _
TriangleTopology, EdgeTopology, Limit As Single, VoxelDescription)

Dim NumberOfCubePoints As Single
Dim Pruseciky(1 To 12) As XYZ ' body v polovine hran topologicky
Dim i As Integer
Dim ReadIndex As Integer
If index = 0 Or index = 255 Then GoTo konec ' zadny nebo zcela zaplneni prostor

' linearni aproxiamce

GeneratePruseciky Pruseciky, StartPoint, EdgeTopology, grid, Limit, VoxelDescription, index

For i = 1 To TriangleTopology(index, 0)
    ReadIndex = (i - 1) * 3
    SetTriangle NumberOfTriangles, Layer, Triangles, _
    TriangleTopology(index, ReadIndex + 1), _
    TriangleTopology(index, ReadIndex + 2), _
    TriangleTopology(index, ReadIndex + 3), _
    Pruseciky
  
Next i

konec:
End Sub

Sub SetTriangle(NumberOfTriangles, _
Layer As Integer, Triangles() As TRIANGLE, b1, b2, b3, Pruseciky() As XYZ)
'Pridani trojuhelniku do pole

NumberOfTriangles = NumberOfTriangles + 1

If NumberOfTriangles > 400000 Then
  MsgBox "Sorry, to complicated scene, Number of reguired triangles > 1 200 000"
  Stop
End If

    Triangles(Layer, NumberOfTriangles).p(0) = Pruseciky(b1)
    Triangles(Layer, NumberOfTriangles).p(1) = Pruseciky(b2)
    Triangles(Layer, NumberOfTriangles).p(2) = Pruseciky(b3)
           
End Sub

Private Sub ReadMarchingTopology(TriangleToplogy, EdgeTopology)
' Cteni pole topologie, zbytecne se provadi vicekrat !!
Dim i As Integer
Dim j As Integer
Dim aa As Integer
Dim KontrolValue As Integer
Dim FullPoint(0 To 255, 0 To 8) As Integer ' 0 pocet bodu, cisla vrcholu
'Dim EdgeToplogy(0 To 255, 0 To 12) As Integer ' 0 pocet hran s vrcholy tr., cisla hran
'Dim TriangleSetup(0 To 255, 0 To 30) As Integer ' Toplogie trojuhelniku
' 0 pocet trojuhelniku, 3 x hrany s vrcholy
Open App.Path & "\" & "cell_table.dat" For Input As #1
'  ocislovani a vrcholy a hrany krychle
'
'                  8--------/7/--------7
'                 /|                  /|
'               /11/                /12/
'               /  |                /  |
'              4---------/3/-------3  /6/
'              |  /8/              |   |
'              |   |               |   |
'              |   |               |   |
'             /4/  5- - - /5/ - - /2/- 6
'              |  /                |   /
'              |/9/                | /10/
'              |/                  | /
'              1--------/1/--------2
' 1 ... 8 cisla vrcholu, /1/ ... /12/ cisla hran

For i = 0 To 255
    For j = 0 To 8
        Input #1, aa
        FullPoint(i, j) = aa
    Next j
    For j = 0 To 12
        Input #1, aa
        EdgeTopology(i, j) = aa
    Next j
    For j = 0 To 30
        Input #1, aa
        TriangleToplogy(i, j) = aa
    Next j
    Input #1, KontrolValue
    If KontrolValue <> -99 Then Stop
Next i

Close #1

End Sub

